import 'package:price_module/framework/model/ds_model.dart';
import 'package:price_module/page/model/details/index.dart';
import 'package:price_module/utils/time_utils.dart';
import 'package:xiaoxiong_repository/entity/product_details_entity.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';
import 'package:xiaoxiong_repository/repository/price_repository.dart';
import 'package:xiaoxiong_repository/entity/turn_chain_entity.dart';
import 'package:xiaoxiong_repository/app_xiaoxiong_repository.dart';

class PriceDetailsService {
  PriceRepository _priceRepository = PriceRepository.get();

  ///
  /// 查询商品详情
  /// [productId]               查询的商品Id
  ///
  Future<PriceDetailsVo> qryProductDetails({required String productId}) async {
    return await _priceRepository
        .qryProductDetails(id: productId)
        .asStream()
        .map((event) {
          PriceDetailsVo? detailsVo;
          if (event.isSuccess) {
            detailsVo = PriceDetailsVo();
            ProductDetailsEntity entity = event.data!;
            detailsVo.viewState = DSViewState.idle;
            detailsVo.likeTimes = entity.likeTimes;
            detailsVo.collegeTimes = entity.collegeTimes;
            detailsVo.itemTitle = entity.itemTitle;
            detailsVo.subTitle = entity.special;
            detailsVo.itemPic = entity.itemPics;
            detailsVo.description = entity.description;
            detailsVo.status = entity.itemStatus;
            detailsVo.buttonText = entity.buttonText;
            detailsVo.itemId = entity.itemId;
            detailsVo.platform = entity.platform;
            detailsVo.couponText = entity.couponText;
            detailsVo.isLike = entity.isLike == '1';
            detailsVo.isFavorites = entity.isCollect == '1';
            detailsVo.shareMessage = entity.shareMessage ?? '';
            detailsVo.shareUrl = entity.shareUrl ?? '';
            if (entity.releaseTime != null && entity.releaseTime!.length > 0) {
              detailsVo.releaseTime =
                  TimeUtils.formatTime(timestamp: entity.releaseTime!);
            }
            detailsVo.checkText = entity.checkText ?? '';
            detailsVo.endPrice = entity.endPrice;
            detailsVo.price = entity.price;
            detailsVo.coupons = List.generate(entity.couponInfos?.length??0, (index) {
              CouponInfo info = entity.couponInfos![index];
              return Coupon(name: info.couponName, url: info.url, status: info.status);
            });
            detailsVo.isOverdue = entity.isOverdue;
            detailsVo.shopTitle = entity.shopTitle;
            detailsVo.rebate = entity.rebate;
            detailsVo.collectOrders = entity.collectOrders;
            detailsVo.newItemInfo = entity.newItemInfo;
          }
          return detailsVo ?? PriceDetailsVo();
        })
        .first
        .catchError((error) {
          print("==================> error:$error");
        });
  }

  ///
  /// 好价推荐列表
  /// [keywords]            搜索关键词
  /// [excludeId]          排除id
  /// [page]                  当前页码
  /// [pagesize]              当前页显示的大小
  ///
  Future<DSResponseList<ProductItemEntity>> getSellingRecommends({
    String? keywords,
    String? excludeId,
    int page = 1,
    int pagesize = 10,
  }) async {
    return _priceRepository.getSellingRecommends(
      keywords: keywords,
      excludeId: excludeId,
      page: page,
      pagesize: pagesize,
    );
  }

  ///
  /// 举报
  /// [id]                  举报的商品Id
  /// [report_type]         举报的商品类型
  ///
  Future<dynamic> report(
      {required String id, required String reportType}) async {
    return await _priceRepository
        .report(id: id, reportType: reportType)
        .asStream()
        .map((event) {
      return {'code': event.code, 'msg': event.msg};
    }).first;
  }

  ///
  /// 用户喜欢
  /// [markId]                  产品Id
  /// [type]                    类型
  /// [status]                  状态
  /// [platform]                平台
  ///
  Future<dynamic> userLike(
      {required String markId,
      required String platform,
      String type = '2',
      String status = '1'}) async {
    return await _priceRepository
        .userLike(
            markId: markId, type: type, status: status, platform: platform)
        .asStream()
        .map((event) {
      return {'code': event.code, 'msg': event.msg};
    }).first;
  }

  ///
  /// 用户收藏
  ///
  Future<dynamic> userCollect(
      {required String itemId,
      required String platform,
      String type = '2',
      String status = '1'}) async {
    return await _priceRepository
        .userCollect(
            itemId: itemId, type: type, status: status, platform: platform)
        .asStream()
        .map((event) {
      return {'code': event.code, 'msg': event.msg};
    }).first;
  }

  ///
  /// 通过商品Id进行转链
  /// [itemId]                  商品id
  /// [platform]                平台，1淘宝 2京东 3拼多多 4唯品会
  /// [type]                    转链类型  好价传6   默认为5
  ///
  Future<DSResponseObject<TurnChainEntity>> turnChainForItemId(
      {required String itemId,
      required String platform,
      String type = '6'}) async {
    return await _priceRepository
        .turnChainForItemId(itemId: itemId, platform: platform, type: type)
        .asStream()
        .first;
  }
}
