import 'dart:math';
import 'dart:ui';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:price_module/page/model/index.dart';
import 'package:price_module/widget/appbar/index.dart';
import 'package:price_module/widget/image/image_widget.dart';
import 'package:price_module/widget/pull/pull_widget.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'other/other_widget.dart';
import 'package:common_module/utils/xapp_utils.dart';

///
/// 好价页面AppBar
/// [onSearchTap]             搜索点击事件
/// [onLayoutTap]             布局变化点击事件
///
Widget buildAppBar({
  required Function onSearchTap,
  required Function(bool) onLayoutTap,
  bool isGrid = false,
}) {
  return buildAppSearchBar(
    height: 44.w,
    padding: EdgeInsets.symmetric(horizontal: 13.w),
    left: Image.asset(
      'assets/images/ic_haojia_logo.png',
      package: 'price_module',
      width: 46.w,
      height: 19.w,
    ),
    child: Container(
      height: 34.w,
      margin: EdgeInsets.symmetric(horizontal: 8.w),
      padding: EdgeInsets.symmetric(horizontal: 13.w),
      decoration: BoxDecoration(
          color: Color(0xFFFFFFFF), borderRadius: BorderRadius.circular(17.w)),
      child: InkWell(
        onTap: onSearchTap as void Function()?,
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Image.asset(
              'assets/images/ic_home_search.png',
              package: 'price_module',
              width: 16.w,
              height: 16.w,
            ),
            Container(
              child: Text(
                '搜索关键词或粘贴商品标题/链接',
                style: TextStyle(
                    fontSize: 13.w,
                    color: Color(0xFFAAAAAA),
                    fontWeight: FontWeight.w400),
              ),
              margin: EdgeInsets.only(left: 8.w),
            )
          ],
        ),
      ),
    ),
    right: InkWell(
      child: Image.asset(
        'assets/images/price_layout_' +
            (isGrid ? 'list' : 'grid') +
            '_icon.png',
        package: 'price_module',
        width: 28.w,
        height: 28.w,
      ),
      onTap: () => onLayoutTap(!isGrid),
    ),
  );
}

///
/// 专区部分单个Widget
/// [image]             单个Widget的图片
///
Widget _buildZoneItemWidget({required String image, void Function()? onTap}) {
  return Expanded(
    child: InkWell(
      child: Container(
        height: 64.w,
        decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(12.w))),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            ImageWidget(
              name: image,
              width: 118.w,
            )
          ],
        ),
      ),
      onTap: onTap,
    ),
  );
}

///
/// 专区Widget
/// [onValueTap]            专区点击事件
///
Widget buildZoneWidget({void Function(String)? onValueTap}) {
  return Container(
      margin: EdgeInsets.symmetric(vertical: 8.w, horizontal: 13.w),
      child: Row(
        children: [
          _buildZoneItemWidget(
            image: 'assets/images/img_baicai_area.png',
            onTap: () {
              if (onValueTap != null) {
                onValueTap('1');
              }
            },
          ),
          SizedBox(
            width: 9,
          ),
          _buildZoneItemWidget(
            image: 'assets/images/img_baokuan_area.png',
            onTap: () {
              if (onValueTap != null) {
                onValueTap('2');
              }
            },
          ),
        ],
      ));
}

///
/// 背景渐变色布局
/// [valueListenable]               数据变化
///
Widget backgroundColorWidget(
    {required ValueListenable<double> valueListenable}) {
  return ValueListenableBuilder<double>(
    valueListenable: valueListenable,
    builder: (BuildContext context, double value, Widget? child) {
      return Positioned(
        child: Container(
          height: 200,
          decoration: BoxDecoration(
              gradient: LinearGradient(
                  colors: [Colors.white, Color(0x00FFFFFF)],
                  begin: Alignment.topCenter,
                  end: Alignment.bottomCenter)),
        ),
        top: value,
        left: 0,
        right: 0,
      );
    },
  );
}

///
/// 回到顶部
/// [valueListenable]         当value为1时显示回到顶部按钮
/// [onPressed]               点击回到顶部按钮
///
Widget backTopWidget(
    {required ValueListenable<int> valueListenable,
    required void Function() onPressed}) {
  return ValueListenableBuilder<int>(
      valueListenable: valueListenable,
      builder: (BuildContext context, dynamic value, Widget? child) {
        if (value == 1) {
          return Positioned(
              bottom: 10.w,
              right: 6.w,
              child: IconButton(
                  icon: Image.asset('assets/all_top_icon.png'),
                  iconSize: 44.w,
                  onPressed: onPressed));
        }
        return Container();
      });
}

Widget buildCustomScroll(
    {required List<Widget> children,
    void Function()? onRefresh,
    void Function()? onLoad,
    RefreshController? refreshController,
    required bool Function(ScrollNotification) onNotification,
    required ScrollController controller}) {
  return NotificationListener(
    onNotification: onNotification,
    child: PullWidget(
      child: CustomScrollView(
        controller: controller,
        cacheExtent: 1000,
        slivers: children,
      ),
      onRefresh: onRefresh,
      controller: refreshController,
      onLoad: onLoad,
    ),
  );
}

///
/// 显示分类的弹窗
/// [structure]                 分类的数据结构
///
Widget _buildClassifyShowWidget(
    {List<dynamic>? structure,
    required void Function(Map<String, dynamic>) onResultTap,
    required void Function() onResetTap,
    Map<String?, dynamic>? selected}) {
  return ClassifyShowWidget(
    selected: selected,
    structure: structure,
    onResetTap: onResetTap,
    onResultTap: onResultTap,
  );
}

///
/// 商城和最新Tab
///
Widget _newestTabWidget({required List<dynamic> structure,void Function(int)? onChange,Map<String?, dynamic>? selected}) {
  return Container(
    decoration: BoxDecoration(color: Colors.white,borderRadius: BorderRadius.vertical(bottom: Radius.circular(10))),
    child: Column(
      children: List.generate(structure.length, (index) {
      dynamic map = structure[index];
      bool enable = false;
      if (selected != null && selected.isNotEmpty) {
        enable = map['type'] == selected['type'];
      }
      return InkWell(
        onTap: () {
          if (onChange != null) {
            onChange(index);
          }
        },
        child: Container(
          alignment: Alignment.centerLeft,
          padding: EdgeInsets.only(left: 30.w),
          height: 44.w,
          child: Text('${map['text']}',style: TextStyle(fontSize: 13.w,color: Color(enable ? 0xFFFF8000 : 0xFF666666)))
        )
      );
      }),
    )
  );
}

///
/// Tab
///
Widget buildTapWidget(BuildContext context,
    {required List<TabVo> tabList,
    void Function(int, bool)? onChange,
    required void Function(int?, TabVo) onNodeChange}) {
  Widget categoryWidget({Widget Function()? onHeaderWidget}) { return Container(
      height: 40.w,
      color: Color(0xFFF5F5F5),
      child: Row(
          children: List<Widget>.generate(tabList.length, (index) {
        TabVo tabVo = tabList[index];
        String? title = getHeaderTitle(tabVo);
        return Expanded(
            child:OtherWidget(
            onHeaderWidget:onHeaderWidget,
            child: Container(
              alignment: Alignment.center,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Container(
                      constraints: BoxConstraints(maxWidth: 65.w),
                      child: Text('$title',
                          style: TextStyle(
                              color: Color(
                                  (tabVo.selected != null || tabVo.open)
                                      ? 0xFFFF8000
                                      : 0xFF999999),
                              fontSize: 13.w),
                          maxLines: 1,
                          overflow: TextOverflow.ellipsis)),
                  Container(
                    margin: EdgeInsets.only(left: 4.w),
                    child: Image.asset(
                      'assets/images/${tabVo.open ? 'ic_arrow_up' : 'ic_arrow_down'}.png',
                      package: 'price_module',
                      width: 6.w,
                      height: 4.w,
                    ),
                  )
                ],
              ),
            ),
            tapUpCallback: (OverlayWidgetBuilder callback) {
              mCallback = callback;
              if (onChange != null) {
                onChange(index, true);
              }
              if (tabVo.type == '3') {
                List<String?> tabs = [];
                tabVo.structure?.forEach((ele) {
                  tabs.add(ele['text']);
                });

                callback(
                  _newestTabWidget(
                    structure: tabVo.structure!,
                    onChange: (v) {
                      if (tabVo.structure != null &&
                          tabVo.structure!.length > v) {
                        dynamic data = tabVo.structure![v];
                        tabVo.selected = data;
                        onNodeChange(index, tabVo);
                        callback(null);
                      }
                    },
                    selected: tabVo.selected,
                  ),
                );
              } else if (tabVo.type == '4') {
                callback(_buildPriceShowWidget(
                  structure: tabVo.structure![0],
                  keyName: tabVo.keyName,
                  onResultTap: (Map<String?, dynamic> data) {
                    tabVo.selected = data;
                    onNodeChange(index, tabVo);
                    callback(null);
                  },
                  onResetTap: () {
                    tabVo.selected = null;
                    onNodeChange(index, tabVo);
                    callback(null);
                  },
                  selected: tabVo.selected ?? {},
                ));
              } else if (tabVo.type == '5') {
                callback(_buildClassifyShowWidget(
                    selected: tabVo.selected,
                    structure: tabVo.structure,
                    onResetTap: () {
                      tabVo.selected = null;
                      onNodeChange(index, tabVo);
                      callback(null);
                    },
                    onResultTap: (Map<String, dynamic> result) {
                      tabVo.selected = result;
                      onNodeChange(index, tabVo);
                      callback(null);
                    }));
              }
            },
            onCloseTap: () {
              if (onChange != null && _jindex != -1) {
                onChange(_jindex, false);
              }
              if (onChange != null) {
                onChange(index, false);
              }
              _jindex = -1;
            },
          ),
        );
      })));}

  return SliverPersistentHeader(
    pinned: true,
    floating: true,
    delegate:
        _SliverTabBarDelegate(maxHeight: 40, minHeight: 40, child: categoryWidget(onHeaderWidget:(){
          if(_jindex is int && _jindex != -1){
            for (var i = 0; i < tabList.length; i++) {
              if(_jindex == i){tabList[i].open = true;}else{tabList[i].open = false;}
            }
          }
          return OverlayEntryHeader(tabList:tabList,onHeaderTitle:(TabVo tabVo){
            return getHeaderTitle(tabVo);
          },onTap:(int jindex,TabVo tabVo){
            _jindex = jindex;
            if(mCallback == null){return;}
            if (onChange != null) {
              onChange(jindex, true);
            }
            onTapUpCallback(callback: mCallback,onChange:onChange,tabVo:tabVo,onNodeChange:onNodeChange,index:jindex);
          },);
        })),
  );
}

///
/// 价格Table
///
Widget _buildPriceShowWidget(
    {required Map<String, dynamic> structure,
    String? keyName,
    required void Function(Map<String?, dynamic>) onResultTap,
    required void Function() onResetTap,
    Map<String?, dynamic>? selected}) {
  TextEditingController controller1 = TextEditingController();
  TextEditingController controller2 = TextEditingController();

  String hintText1 = '';
  String hintText2 = '';

  String keyName1 = '';
  String keyName2 = '';
  try {
    if (selected != null && selected['price_section'] is Map) {
      selected = Map<String, dynamic>.from(selected['price_section']);
    }
    if (structure['label'] != null && structure['label'] is List) {
      List<dynamic> labels = List<dynamic>.from(structure['label']);
      if (labels.length > 0) {
        hintText1 = '${labels[0]['text']}';
        keyName1 = '${labels[0]['key_name']}';
      }
      if (labels.length > 1) {
        hintText2 = '${labels[1]['text']}';
        keyName2 = '${labels[1]['key_name']}';
      }
    }
    if (selected![keyName1] is String) {
      controller1.text = '${selected[keyName1]}';
    }
    if (selected[keyName2] is String) {
      controller2.text = '${selected[keyName2]}';
    }
  } catch (e) {
    print(e);
  }
  return Container(
    width: double.infinity,
    decoration: BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.vertical(bottom: Radius.circular(10))),
    child: Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        // 标题
        Container(
          child: Text(
            '${structure['text']}',
            style: TextStyle(
                color: Color(0xFF333333),
                fontSize: 13,
                fontWeight: FontWeight.w500),
          ),
          margin: EdgeInsets.only(top: 12, left: 13),
        ),

        //价格输入
        Container(
            margin: EdgeInsets.only(left: 13, right: 13, top: 12),
            child: Row(children: [
              Expanded(
                child: Container(
                    alignment: Alignment.center,
                    height: 44,
                    padding: EdgeInsets.only(left: 48.w),
                    decoration: BoxDecoration(
                        border: Border(
                            bottom: BorderSide(
                                width: 0.5.w, color: Color(0xFFEDEDED)))),
                    child: TextField(
                      controller: controller1,
                      keyboardType:
                          TextInputType.numberWithOptions(decimal: true),
                      textAlign: TextAlign.left,
                      inputFormatters: [
                        FilteringTextInputFormatter.allow(RegExp(r'[0-9.]'))
                      ],
                      cursorColor: Color(0xFFFFCA00),
                      decoration: InputDecoration(
                          hintText: hintText1,
                          hintStyle: TextStyle(
                              fontSize: 13.w, color: Color(0xFF999999)),
                          border: InputBorder.none),
                      style:
                          TextStyle(fontSize: 13.w, color: Color(0xFF333333)),
                    )),
                flex: 1,
              ),
              Container(
                width: 69.w,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Container(
                      alignment: Alignment.center,
                      width: 10.w,
                      height: 2.w,
                      color: Color(0xFF333333),
                    )
                  ],
                ),
              ),
              Expanded(
                child: Container(
                  alignment: Alignment.center,
                  height: 44.w,
                  padding: EdgeInsets.only(left: 48.w),
                  decoration: BoxDecoration(
                      border: Border(
                          bottom: BorderSide(
                              width: 0.5.w, color: Color(0xFFEDEDED)))),
                  child: TextField(
                      controller: controller2,
                      keyboardType:
                          TextInputType.numberWithOptions(decimal: true),
                      textAlign: TextAlign.left,
                      inputFormatters: [
                        FilteringTextInputFormatter.allow(RegExp(r'[0-9.]'))
                      ],
                      decoration: InputDecoration(
                        border: InputBorder.none,
                        hintText: hintText2,
                        hintStyle:
                            TextStyle(fontSize: 13.w, color: Color(0xFF999999)),
                      ),
                      cursorColor: Color(0xFFFFCA00),
                      style:
                          TextStyle(fontSize: 13.w, color: Color(0xFF333333))),
                ),
                flex: 1,
              ),
            ])),

        //底部按钮
        Container(
          margin: EdgeInsets.only(left: 13, right: 13, bottom: 8, top: 28),
          child: Row(
            children: [
              Expanded(
                child:InkWell(
                      child: Container(
                        alignment: Alignment.center,
                        height: 40,
                        child: Text('重置',
                            style: TextStyle(
                                fontSize: 15, fontWeight: FontWeight.w500)),
                        decoration: BoxDecoration(
                            color: Color(0xFFF5F5F5),
                            borderRadius: BorderRadius.circular(8)),
                      ),
                      onTap: () {
                        onResetTap();
                      }),
                flex: 1,
              ),
              SizedBox(width: 12),
              Expanded(
                child:InkWell(
                      child: Container(
                        alignment: Alignment.center,
                        height: 40,
                        child: Text('确认',
                            style: TextStyle(
                                fontSize: 15, fontWeight: FontWeight.w500)),
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(8),
                            color: Color(0xFFFFBB00)),
                      ),
                      onTap: () {
                        Map<String, dynamic> resultMap = {};
                        String lowest = controller1.text.trim();
                        String highest = controller2.text.trim();
                        if (keyName1.isNotEmpty && lowest.isNotEmpty) {
                          resultMap[keyName1] = lowest;
                        }
                        if (keyName2.isNotEmpty && highest.isNotEmpty) {
                          resultMap[keyName2] = highest;
                        }
                        if (resultMap.isEmpty) {
                          onResetTap();
                        } else {
                          Map<String?, dynamic> result = {'keyName': keyName,keyName: resultMap};
                          onResultTap(result);
                        }
                      }
                    ),
                 
                flex: 3,
              ),
            ],
          ),
        )
      ],
    ),
  );
}

Widget _buildNewestWidget(
    {required List<dynamic> structure,
    required void Function(int) onChange,
    Map<String?, dynamic>? selected}) {
  return Container(
    decoration: BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.vertical(bottom: Radius.circular(10))),
    child: ListView.builder(
      padding: EdgeInsets.zero,
      shrinkWrap: true,
      itemCount: structure.length,
      physics: NeverScrollableScrollPhysics(),
      itemBuilder: (BuildContext context, int index) {
        dynamic map = structure[index];
        bool enable = false;
        if (selected != null && selected.isNotEmpty) {
          enable = map['type'] == selected['type'];
        }
        return InkWell(
          child: Container(
            alignment: Alignment.centerLeft,
            padding: EdgeInsets.only(left: 30.w),
            height: 44.w,
            child: Text("${map['text']}",
                style: TextStyle(
                    fontSize: 13.w,
                    color: Color(enable ? 0xFFFF8000 : 0xFF666666))),
          ),
          onTap: () {
            onChange(index);
          },
        );
      },
    ),
  );
}

String? getHeaderTitle(TabVo tabVo ){
  String? title = '';
    if (tabVo.selected == null) {
      title = tabVo.text;
    } else if (tabVo.keyName == 'price_section') {
      try {
        if (tabVo.selected!['price_section'] is Map) {
          Map<String, dynamic>.from(tabVo.selected!['price_section'])
              .values
              .forEach((ele) {
            if (title!.isNotEmpty) {
              title = title! + '-';
            }
            title = title! + '$ele';
          });
        }
      } catch (e) {
        title = tabVo.text;
      }
    } else {
      title = '${tabVo.selected!['text'] ?? tabVo.selected!['name']}';
    }
  return title;
}

void onTapUpCallback({OverlayWidgetBuilder? callback,void Function(int, bool)? onChange,required TabVo tabVo,void Function(int?, TabVo)? onNodeChange,int? index}){
  if (tabVo.type == '3') {
              callback!(_buildNewestWidget(
                  structure: tabVo.structure ?? [],
                  onChange: (v) {
                    if (tabVo.structure != null && tabVo.structure!.length > v) {
                      callback(null);
                      dynamic data = tabVo.structure![v];
                      tabVo.selected = data;
                      if (onNodeChange != null) {
                        onNodeChange(index, tabVo);
                      }
                    }
                  },
                  selected: tabVo.selected));
            } else if (tabVo.type == '4') {
              callback!(_buildPriceShowWidget(
                structure: tabVo.structure![0],
                keyName: tabVo.keyName,
                onResultTap: (Map<String?, dynamic> data) {
                  tabVo.selected = data;
                  if (onNodeChange != null) {
                    onNodeChange(index, tabVo);
                  }
                  callback(null);
                },
                onResetTap: () {
                  tabVo.selected = null;
                  if (onNodeChange != null) {
                    onNodeChange(index, tabVo);
                  }
                  callback(null);
                },
                selected: tabVo.selected ?? {},
              ));
            } else if (tabVo.type == '5') {
              callback!(_buildClassifyShowWidget(
                  selected: tabVo.selected,
                  structure: tabVo.structure,
                  onResetTap: () {
                    tabVo.selected = null;
                    if (onNodeChange != null) {
                      onNodeChange(index, tabVo);
                    }
                    callback(null);
                  },
                  onResultTap: (Map<String, dynamic> result) {
                    tabVo.selected = result;
                    if (onNodeChange != null) {
                      onNodeChange(index, tabVo);
                    }
                    callback(null);
                  }));
            }
}

OverlayWidgetBuilder? mCallback;
int _jindex = -1;

///
/// 分类
///
Widget buildCategoryTapWidget(List<TabVo> tabList, List<dynamic> sellingTabs,
    {GlobalKey? anchorKey,
    Map<String, dynamic>? sellTab,
    void Function(Map<String, dynamic>)? onChangeSort,
    required void Function(int, bool) onChange,
    required void Function(int?, TabVo) onNodeChange}) {
  Widget categoryWidget(int index,{Widget Function()? onHeaderWidget}) {
    TabVo tabVo = tabList[index];
    String? title = getHeaderTitle(tabVo);
    return Expanded(
        child: OtherWidget(
          onHeaderWidget:onHeaderWidget,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Container(
                  constraints: BoxConstraints(maxWidth: 65.w),
                  margin: EdgeInsets.only(right: 4.w),
                  child: Text('$title',
                      style: TextStyle(
                          color: Color((tabVo.selected != null || tabVo.open)
                              ? 0xFFFF8000
                              : 0xFF302600),
                          fontSize: 13.w),
                      maxLines: 1,
                      overflow: TextOverflow.ellipsis)),
              Image.asset(
                'assets/images/${tabVo.open ? 'ic_arrow_up' : 'ic_arrow_down'}.png',
                package: 'price_module',
                width: 6.w,
                height: 4.w,
              )
            ],
          ),
          onCloseTap: () {
            if (_jindex != -1) {
              onChange(_jindex, false);
            }
            onChange(index, false);
            _jindex = -1;
          },
          tapUpCallback: (OverlayWidgetBuilder callback) {
            mCallback = callback;
            onChange(index, true);
            onTapUpCallback(callback: mCallback,onChange:onChange,tabVo:tabVo,onNodeChange:onNodeChange,index:index);
          },
        ),
        flex: 1);
  }

  return SliverPersistentHeader(
      pinned: true,
      floating: true,
      delegate: _SliverTabBarDelegate(
          maxHeight: 64.w + 12.w,
          minHeight: 64.w + 12.w,
          child: Container(
            key: anchorKey,
            decoration: BoxDecoration(
                color: Color(0xFFFFFFFF),
                borderRadius:
                    BorderRadius.vertical(top: Radius.circular(12.0))),
            child: Column(
              children: [
                Container(
                    height: 40.w,
                    child: Row(children:List.generate(tabList.length, (index)=>categoryWidget(
                      index,
                      onHeaderWidget:(){
                        if(_jindex is int && _jindex != -1){
                          for (var i = 0; i < tabList.length; i++) {
                            if(_jindex == i){
                              tabList[i].open = true;
                            }else{
                              tabList[i].open = false;
                            }
                          }
                        }
                        return OverlayEntryHeader(tabList:tabList,onHeaderTitle:(TabVo tabVo){
                          return getHeaderTitle(tabVo);
                        },onTap:(int jindex,TabVo tabVo){
                          _jindex = jindex;
                          if(mCallback == null){return;}
                          onChange(jindex, true);
                          onTapUpCallback(callback: mCallback,onChange:onChange,tabVo:tabVo,onNodeChange:onNodeChange,index:jindex);
                        },);
                      })
                    ))
                ),
                Container(
                  height: 24.w + 12.w,
                  child: ListView.builder(
                    itemCount: sellingTabs.length,
                    padding: EdgeInsets.only(left: 5.w, right: 13.w),
                    scrollDirection: Axis.horizontal,
                    itemBuilder: (BuildContext context, int index) {
                      Map<String, dynamic> sellingMap = sellingTabs[index];
                      bool enable = false;
                      if (sellTab != null) {
                        enable = sellingMap['id'] == sellTab['id'];
                      }
                      return InkWell(
                        child: Container(
                            color: Colors.transparent,
                            child: Column(
                              children: [
                                Container(
                                    height: 24.w,
                                    margin: EdgeInsets.only(left: 8.w),
                                    padding:
                                        EdgeInsets.symmetric(horizontal: 10.w),
                                    alignment: Alignment.center,
                                    decoration: BoxDecoration(
                                        color: Color(
                                            enable ? 0x1AFF8000 : 0xFFF5F5F5),
                                        borderRadius: BorderRadius.all(
                                            Radius.circular(4.w))),
                                    child: Text(
                                      '${sellingMap['name']}',
                                      style: TextStyle(
                                          color: Color(
                                              enable ? 0xFFFF8000 : 0xFF666666),
                                          fontSize: 12.w),
                                    ))
                              ],
                            )),
                        onTap: () {
                          if (onChangeSort != null) {
                            if (enable) {
                              onChangeSort({});
                            } else {
                              onChangeSort(sellingMap);
                            }
                          }
                        },
                      );
                    },
                  ),
                )
              ],
            ),
          )));
}

///
/// 当搜索的商品数据为空
///
Widget buildEmptyProductList({bool refresh = false, void Function()? onResetTap}) {
  Widget widget = Container(
    margin: EdgeInsets.only(top: 120.w),
    child: Column(
      children: [
        ImageWidget(name: 'assets/images/img_ empty_data.png', width: 168),
        Container(child: Text('啥也没有空空如也～',style: TextStyle(fontSize: 14, color: Color(0xFF999999))),margin: EdgeInsets.symmetric(vertical: 12)),
        InkWell(
          onTap: onResetTap,
          child: Container(
            width: 160,
            height: 40,
            alignment: Alignment.center,
            child: Text(refresh ? '点击刷新' : '重置筛选',style: TextStyle(color: Color(0xFF3D3101),fontSize: 16,fontWeight: FontWeight.w500)),
            decoration: BoxDecoration(borderRadius: BorderRadius.all(Radius.circular(20)),color: Color(0xFFFFCA00)),
          )
        ),
      ],
    )
  );
  return widget;
}

class _SliverTabBarDelegate extends SliverPersistentHeaderDelegate {
  _SliverTabBarDelegate({
    required this.minHeight,
    required this.maxHeight,
    required this.child,
  });

  final double minHeight;
  final double maxHeight;
  final Widget child;

  @override
  double get minExtent => minHeight;

  @override
  double get maxExtent => max(maxHeight, minHeight);

  @override
  Widget build(
      BuildContext context, double shrinkOffset, bool overlapsContent) {
    return new SizedBox.expand(child: child);
  }

  @override
  bool shouldRebuild(_SliverTabBarDelegate oldDelegate) {
    return maxHeight != oldDelegate.maxHeight ||
        minHeight != oldDelegate.minHeight ||
        child != oldDelegate.child;
  }
}

///
/// 分类Widget
///
class ClassifyShowWidget extends StatefulWidget {
  final Map<String?, dynamic>? selected;
  final void Function(Map<String, dynamic>) onResultTap;
  final void Function() onResetTap;
  final List<dynamic>? structure;

  const ClassifyShowWidget(
      {Key? key,
      this.structure,
      required this.onResultTap,
      required this.onResetTap,
      this.selected})
      : super(key: key);

  @override
  _ClassifyShowWidgetState createState() => _ClassifyShowWidgetState();
}

class _ClassifyShowWidgetState extends State<ClassifyShowWidget> {
  List<dynamic>? structure, structureNode1;

  int childNode1 = 0, childNode2 = 0;

  @override
  void initState() {
    this.structure = widget.structure;
    try {
      if (widget.selected != null && widget.selected!.isNotEmpty) {
        String? level = widget.selected!['level'];
        String? parentId = widget.selected!['parent_id'];
        String? id = widget.selected!['id'];
        if (level == '1') {
          childNode1 = this.structure!.indexWhere((ele) => (ele['id'] == id));
        } else if (level == '2') {
          childNode1 =
              this.structure!.indexWhere((ele) => (ele['id'] == parentId));
          if (this.structure![childNode1] != null) {
            childNode2 = List<dynamic>.from(Map<String, dynamic>.from(
                    this.structure![childNode1])['child'])
                .indexWhere((ele) => (ele['id'] == id));
          }
        }
      }
    } catch (e) {}
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    if (structure![childNode1]['child'] != null) {
      structureNode1 = List<dynamic>.from(structure![childNode1]['child']);
    }
    return Container(
      alignment: Alignment.topCenter,
      constraints: BoxConstraints(maxHeight: 470.w),
      decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.vertical(bottom: Radius.circular(10))),
      child: Column(
        children: [
          Expanded(
            child: Container(
                child: Row(
              children: [
                Container(
                  width: 94.w,
                  padding: EdgeInsets.all(0),
                  child: ListView.builder(
                      itemCount: structure!.length,
                      padding: EdgeInsets.all(0),
                      itemBuilder: (BuildContext context, int index) {
                        dynamic dMap = structure![index];
                        return InkWell(
                          child: Container(
                              padding: EdgeInsets.only(left: 12.w),
                              height: 44.w,
                              child: Text(
                                '${dMap['name']}',
                                maxLines: 1,
                                overflow: TextOverflow.ellipsis,
                                style: TextStyle(
                                    fontWeight: childNode1 == index
                                        ? FontWeight.w500
                                        : FontWeight.w400,
                                    fontSize: 13.w,
                                    color: Color(childNode1 == index
                                        ? 0xFFFF8000
                                        : 0xFF666666)),
                              ),
                              alignment: Alignment.centerLeft,
                              decoration: BoxDecoration(
                                  color: childNode1 == index
                                      ? Colors.white
                                      : Color(0xFFF5F5F5),
                                  borderRadius: BorderRadius.only(
                                      topRight: Radius.circular(
                                          childNode1 + 1 == index ? 6.w : 0),
                                      bottomRight: Radius.circular(
                                          childNode1 - 1 == index ? 6.w : 0))),
                            ),
                          onTap: () {
                            setState(() {
                              childNode1 = index;
                              childNode2 = 0;
                            });
                          },
                        );
                      }),
                ),
                if (structureNode1 != null && structureNode1!.length > 0)
                  Expanded(
                    child: Container(
                      color: Colors.white,
                      child: GridView.builder(
                        padding: EdgeInsets.all(0),
                        gridDelegate:SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: 2,crossAxisSpacing: 1.w,childAspectRatio: 3.3.w),
                        itemCount: structureNode1!.length,
                        itemBuilder: (BuildContext context, int index) {
                          dynamic dMap = structureNode1![index];
                          return InkWell(
                            child: Container(
                              padding: EdgeInsets.only(left: 24.w),
                              height: 44.w,
                              child: Text('${dMap['name']}',maxLines: 1,overflow: TextOverflow.ellipsis,style: TextStyle(fontSize: 13.w,fontWeight: childNode2 == index ? FontWeight.w500 : FontWeight.w400,color: Color(childNode2 == index? 0xFFFF8000: 0xFF666666))),
                              alignment: Alignment.centerLeft,
                            ),
                            onTap: () {
                              setState(() {childNode2 = index;});
                            }
                          );
                        }
                      ),
                    ),
                  flex: 1
                )
              ],
            )),
            flex: 1,
          ),
          //底部按钮
          Container(
            margin: EdgeInsets.only(left: 13, right: 13, bottom: 8, top: 8),
            child: Row(
              children: [
                Expanded(
                  child:InkWell(
                          child: Container(
                            alignment: Alignment.center,
                            height: 40.w,
                            child: Text('重置',
                                style: TextStyle(
                                    fontSize: 15.w,
                                    fontWeight: FontWeight.w500)),
                            decoration: BoxDecoration(
                                color: Color(0xFFF5F5F5),
                                borderRadius: BorderRadius.circular(8.w)),
                          ),
                          onTap: () {
                            widget.onResetTap();
                          }),
                     
                  flex: 1,
                ),
                SizedBox(width: 12.w),
                Expanded(
                  child:InkWell(
                    child: Container(
                      alignment: Alignment.center,
                      height: 40.w,
                      child: Text('确认',style: TextStyle(fontSize: 15.w,fontWeight: FontWeight.w500)),
                      decoration: BoxDecoration(borderRadius: BorderRadius.circular(8),color: Color(0xFFFFBB00)),
                    ),
                    onTap: () {
                      if (structureNode1 != null && structureNode1!.isNotEmpty) {
                        Map<String, dynamic> result = {};
                        if (childNode2 == 0) {
                          result = Map<String, dynamic>.from(structure![childNode1]);
                        } else {
                          result = Map<String, dynamic>.from(structureNode1![childNode2]);
                        }
                        result.remove('child');
                        widget.onResultTap(result);
                      }
                    }),
                  flex: 3,
                ),
              ],
            ),
          )
        ],
      ),
    );
  }
}
